#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <omp.h>

int main ( int argc, char *argv[] )  
{
    int i, n;
    long long int *fib;

    if (argc != 2)
    {
        fprintf(stderr, "Usage %s <n>\n", argv[0]);
        exit(-1);
    }
    n = atoi(argv[1]);

    fib = malloc(n * sizeof(long long int));

    // Initialize
    fib[0] = 1;
    fib[1] = 1;

    double phi = (1 + sqrt(5)) / 2;
    double psi = (1 - sqrt(5)) / 2;

    //#pragma omp parallel private(i)
    //{
    //  int is_first_two = 2;
    //  #pragma omp for schedule(static)
    //  for (int i = 2; i < n; i++) {
    //    if (is_first_two > 0) {
    //      fib[i] = ((pow(phi, i+1) - pow(psi, i+1)) / sqrt(5));
    //      is_first_two -= 1;
    //    } else {
    //      fib[i] = fib[i - 1] + fib[i - 2];
    //    }
    //  }
    //}

    // Compute
    #pragma omp parallel
    {
      int tid = omp_get_thread_num();
      int nthr = omp_get_num_threads();
      int chunk = (n - 2) / nthr;
      int ifrom = 2 + tid * chunk;
      int ito = ifrom + chunk;
      if (tid == nthr - 1) ito = n;
      fib[ifrom] = (int)((pow(phi, ifrom + 1) - pow(psi, ifrom + 1)) / sqrt(5));
      fib[ifrom + 1] = (int)((pow(phi, ifrom + 2) - pow(psi, ifrom + 2)) / sqrt(5));
      for( i = ifrom + 2; i < ito; i++ )
          fib[i] = fib[i-1] + fib[i-2];
    }

    // Print out the n-th Fibonacci number
    // Do not delete this printf statement
    printf("Fib(%d): %lld\n", n, fib[n-1]);

    free(fib);

    return 0;
}
